"""Naive tester console application

A simple testing system based on files. You provide a name of a script
to be tested and a folder containing files with arguments and reference
outputs. The app runs the script with the arguments from a file and compares
its result with that from the corresponding reference output file.
The following naming convention is applied:
each set of arguments is stored in the '.in' file, while the corresponding
output in the '.out' file, i.e., 'test_0.in' and 'test_0.out'.
Each pair must have identical names except for extensions.

For more information visit https://github.com/FilippSolovev/naive-tester

"""

import logging

logging.getLogger(__name__).addHandler(logging.NullHandler())
logging.basicConfig(level=logging.INFO, format='%(message)s')

# Version of naive_tester package
__version__ = '1.0.0'



"""Handle files for tester"""
import sys
import os
import logging

logger = logging.getLogger(__name__)


def get_files(path, extension='.txt'):
    """Returns a list of files in the path directory based on provided extension.
    """
    reference_files = []
    for dirname, _, files in os.walk(path):
        for fname in files:
            if fname.endswith(extension):
                reference_files.append(os.path.join(dirname, fname))
    # logger.info(f'total {len(reference_files)} files')
    return sorted(reference_files)


def check_files_existence(file_name, err_msg='File does not exit'):
    """Checks if a file or directory exists."""
    if not os.path.exists(file_name):
        logger.error(err_msg)
        sys.exit()


def do_files_comply(actual_in_files, actual_out_files):
    """Check if the 'in' files correspond to the 'out' files by names."""
    expected = set(map(lambda x: x[:-3] + '.out', actual_in_files))
    return not expected.difference(set(actual_out_files))



"""Naive Tester

Usage:
-----

    $ tester [script] [test_case_folder]

script - a script to test
test_case_folder - a folder containing pairs of .in and .out files,
where
    .in holds [script] argument
    .out holds expected output

Contact:
-------

More information is available at:
- https://github.com/FilippSolovev/naive-tester

Version:
-------

- naive-tester v1.0.0
"""
import sys
import logging

from tester.io import check_files_existence, get_files, do_files_comply
from tester.runner import run_jobs

logger = logging.getLogger(__name__)


def main():
    if len(sys.argv) > 1:
        script_name = sys.argv[1]
        dir_name = sys.argv[2]
    else:
        sys.exit()

    check_files_existence(script_name, 'Script given does not exist')
    check_files_existence(script_name, 'Directory given does not exist')

    jobs_arguments = get_files(dir_name, '.in')
    output_files = get_files(dir_name, '.out')
    if not do_files_comply(jobs_arguments, output_files):
        logger.error('Input and output files do not comply')
        sys.exit()

    reference_outputs = []
    for file_name in output_files:
        with open(file_name, 'r') as output_file:
            output = output_file.read().strip()
            reference_outputs.append(output)

    jobs = zip([script_name for _ in jobs_arguments],
               jobs_arguments)

    jobs_with_outputs = zip(jobs, reference_outputs)

    run_jobs(jobs_with_outputs)


if __name__ == '__main__':
    main()



"""Setup script for naive-tester package"""
import os.path
from setuptools import setup


HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, 'README.md')) as file:
    README = file.read()

setup(
    name='naive-tester',
    version='1.0.0',
    description='A simple testing system based on files',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/FilippSolovev/naive-tester',
    author='Filipp Solovev',
    author_email='solovyev.filipp@gmail.com',
    licence='MIT',
    packages=['tester'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent'
    ],
    python_requires='>=3.6',
    entry_points={"console_scripts": ["tester=tester.__main__:main"]},
)



"""Run an external script and assess its output"""
import sys
import subprocess
import logging

logger = logging.getLogger(__name__)


def run_job(job):
    """Run a script with it's arguments.

    Parameters
    ----------
    job : a tuple of script's name and it's arguments.

    Returns
    --------
    Standard out of a script in a string form.

    """
    script, args = job
    output = subprocess.run(
        [sys.executable] + [script, args],
        stdout=subprocess.PIPE,
        universal_newlines=True).stdout.strip('\n')
    return output


def assert_job(job, reference_output):
    """Check if script's output coincides with the reference output."""
    script, args = job
    try:
        assert run_job(job) == reference_output
        logger.info(f'Successfully run the {script} with {args}')
    except AssertionError:
        logger.error(
            f'Failed run of the {script} with {args}. '
            f'The output should be {reference_output}.')


def run_jobs(jobs_with_outputs):
    """Sequentially run scripts."""
    for job_with_output in jobs_with_outputs:
        assert_job(*job_with_output)
